package labss.lang.il;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import labss.lang.il.builders.DescendantOrSelfEntityBuilder;
import labss.lang.il.builders.EntityBuilder;
import labss.lang.il.builders.Event2EntityBuilderAdapter;
import labss.lang.il.builders.ILBuilder;
import labss.lang.il.builders.ILEventBuilderAdapter;
import labss.lang.il.builders.ILModelBuilder;
import labss.lang.il.factories.ILAbstractFactory;
import labss.lang.il.factories.ILPrototypeManager;
import labss.lang.il.factories.ILTemplateManager;
import labss.lang.il.factories.PrototypeManager;
import labss.lang.il.factories.TemplateFactory;
import labss.lang.il.factories.TemplateManager;
import labss.lang.il.iterators.DescendantOrSelfIterator;
import labss.lang.il.model.ILEntity;
import labss.lang.il.model.ILTypesEnum;
import labss.lang.il.model.Literal;
import labss.lang.il.model.Statement;
import labss.lang.il.model.impl.Variable;
import labss.lang.il.visitors.GenericPrettyPrintTypeSwitchBased;
import labss.lang.il.visitors.IndentedPrettyPrintVisitor;
import labss.lang.il.visitors.InterpreterVisitor;
import labss.lang.il.visitors.PrettyPrintTypeSwitchBased;
import labss.lang.il.visitors.PrettyPrintVisitor;
import labss.lang.il.visitors.TopDownVisitor;
import labss.lang.il.visitors.UsedVariablesVisitor;

public class EsempioFattoriale {
	public static Statement creaProgrammaFattoriale(ILAbstractFactory f) {
		Literal createInt = f.createInt(4);
		return f.createBlock(
				f.createAssignment("n", createInt),
				f.createAssignment("result", f.createInt(1)),
				f.createWhile(f.createGreaterThan(f.createVariable("n"), f.createInt(0)), f.createBlock(
						f.createAssignment("result",
								f.createMultiplication(f.createVariable("result"), f.createVariable("n"))),
						f.createAssignment("n",
								f.createSubtraction(f.createVariable("n"), f.createInt(1)))
				)));
	}

	public static void directorProgrammaFattoriale(ILBuilder b) {
		b.Block_();
			b.Assignment_();
				b.Variable("n");
				b.Int(4);
			b._Assignment();
			
			b.Assignment_();
				b.Variable("result");
				b.Int(1);
			b._Assignment();
			
			b.While_();
				b.GreaterThan_();
					b.Variable("n");
					b.Int(0);
				b._GreaterThan();
				
				b.Block_();
					b.Assignment_();
						b.Variable("result");
						b.Multiplication_();
							b.Variable("result");
							b.Variable("n");
						b._Multiplication();
					b._Assignment();
					
					b.Assignment_();
						b.Variable("n");
						b.Subtraction_();
							b.Variable("n");
							b.Int(1);
						b._Subtraction();
					b._Assignment();
				b._Block();
			b._While();
		b._Block();
	}

	private static TemplateFactory creaTemplateProgrammaFattoriale() {
		return new TemplateFactory() {
			public ILEntity create(Object... args) {
				int val = (Integer) args[0];
				
				ILAbstractFactory f = ILAbstractFactory.instance;
				
				return f.createBlock(
						f.createAssignment("n", f.createInt(val)),
						f.createAssignment("result", f.createInt(1)),
						f.createWhile(f.createGreaterThan(f.createVariable("n"), f.createInt(0)), f.createBlock(
								f.createAssignment("result",
										f.createMultiplication(f.createVariable("result"), f.createVariable("n"))),
								f.createAssignment("n",
										f.createSubtraction(f.createVariable("n"), f.createInt(1)))
				)));
			}
		};
	}
	
	public static void main(String[] args) {
		Statement fatt = creaProgrammaFattoriale(ILAbstractFactory.instance);
		
		ILModelBuilder modelBuilder = new ILModelBuilder();
		directorProgrammaFattoriale(modelBuilder);
		ILEntity fatt2 = modelBuilder.getResult();
		System.out.println("\n*** prettyPrint di fattoriale (costruito con builder)");
		IndentedPrettyPrintVisitor.prettyPrint(fatt2, System.out);

		EntityBuilder entityBuilder = new DescendantOrSelfEntityBuilder();
		directorProgrammaFattoriale(
				new ILEventBuilderAdapter(
						new Event2EntityBuilderAdapter(
								entityBuilder)));
		ILEntity fatt3 = entityBuilder.getResult();
		System.out.println("\n*** prettyPrint di fattoriale (costruito con builer e adapters)");
		IndentedPrettyPrintVisitor.prettyPrint(fatt3, System.out);
		
		//aggiungo prototipo del programma fattoriale (con n fisso)
		PrototypeManager pm = ILPrototypeManager.instance();
		pm.put("fatt", fatt);

		System.out.println("\n*** prettyPrint di prototipi");
		IndentedPrettyPrintVisitor.prettyPrint(pm.create("While"), System.out);
		IndentedPrettyPrintVisitor.prettyPrint(pm.create("fatt"), System.out);

		//aggiungo template del programma fattoriale (con parametro n)
		TemplateManager tm = ILTemplateManager.instance();
		tm.put("fattorialeAggiuntoDinamicamente", creaTemplateProgrammaFattoriale());

		System.out.println("\n*** prettyPrint di template");
		IndentedPrettyPrintVisitor.prettyPrint(
				tm.create("fattorialeAggiuntoDinamicamente", 5), System.out);
		IndentedPrettyPrintVisitor.prettyPrint(
				tm.create("While",
						tm.create("GreaterThan", tm.create("Variable", "n"), tm.create("Literal", 0)),
						tm.create("Empty")), System.out);

		
		System.out.println("\n*** prettyPrint visitor (indented)");
		IndentedPrettyPrintVisitor.prettyPrint(fatt, System.out);
		System.out.println("\n*** prettyPrint visitor");
		PrettyPrintVisitor.prettyPrint(fatt, System.out);
		System.out.println("\n*** prettyPrint type switch");
		PrettyPrintTypeSwitchBased.prettyPrint(fatt, System.out);
		System.out.println("\n*** prettyPrint generic type switch");
		GenericPrettyPrintTypeSwitchBased.prettyPrint(fatt, System.out);

		fatt = (Statement) fatt.clone();
		
		System.out.println("\n*** execute visitor");
		System.out.println("n! = " +
				InterpreterVisitor.interpret(fatt).getValue("result")
		);
		
		System.out.println("\n*** node types in DescendantOrSelf order");
		Iterator<ILEntity> i = new DescendantOrSelfIterator(fatt);
		while (i.hasNext()) {
			System.out.println("node: "+i.next().getType().name);
		}
		
		fatt.accept(new TopDownVisitor());
		
		System.out.println("\n*** used variables: (visitor)");
		fatt.accept(new UsedVariablesVisitor());

		System.out.println("\n*** used variables: (iterator)");
		Set<String> vars = new HashSet<String>();
		i = new DescendantOrSelfIterator(fatt);
		while (i.hasNext()) {
			ILEntity e = i.next();
			if (e.getType().equals(ILTypesEnum.tVariable) &&
					vars.add(((Variable) e).getName()))
					System.out.println("var: "+((Variable) e).getName());
		}
	}
}
