package labss.lang.il.builders;

import java.util.ArrayList;
import java.util.List;

import labss.lang.il.model.ILEntity;

/**
 * @author Riccardo Solmi
 */
public class DescendantOrSelfEntityBuilder implements EntityBuilder {
	protected List<EntityBuilder> builderStack = new ArrayList<EntityBuilder>(16);

    public DescendantOrSelfEntityBuilder() {
    	pushBuilder(new SelfEntityBuilder());
    }
    public DescendantOrSelfEntityBuilder(ILEntity entity) {
    	pushBuilder(new SelfEntityBuilder(entity));
    }

    protected EntityBuilder createChildEntityBuilder(ILEntity entity) {
    	return new ChildEntityBuilder(entity);
    }
    protected final EntityBuilder peekBuilder() {
    	return builderStack.get(builderStack.size()-1);
    }
    protected final void pushBuilder(EntityBuilder builder) {
    	builderStack.add(builder);
    }
    protected final EntityBuilder popBuilder() {
    	return builderStack.remove(builderStack.size()-1);
    }

	public ILEntity getResult() {
		return peekBuilder().getResult();
	}

    public void build(ILEntity entity) {
    	peekBuilder().build(entity);
    }
    public void startBuild(ILEntity entity) {
    	peekBuilder().startBuild(entity);
    	pushBuilder(createChildEntityBuilder(entity));
    }
    public void endBuild() {
    	peekBuilder().endBuild();
    	popBuilder();
    }
}
