package labss.lang.il.factories;

import labss.lang.il.model.Expression;
import labss.lang.il.model.ILEntity;
import labss.lang.il.model.Statement;

/**
 * @author Riccardo Solmi
 */
public class ILTemplateManager extends TemplateManager {
	private static ILTemplateManager instance;
	public static ILTemplateManager instance() {
		if (instance == null)
			instance = new ILTemplateManager();
		return instance;
	}

	protected ILTemplateManager() {
		final ILAbstractFactory lf = ILAbstractFactory.instance; 
		put("Empty", new TemplateFactory() {
			public ILEntity create(Object... args) {
				return lf.createEmpty();
			}
		});
		put("While", new TemplateFactory() {
			public ILEntity create(Object... args) {
				//check preconditions
				assert args != null && args.length == 2 &&
					args[0] instanceof Expression &&
					args[1] instanceof Statement;

				//oppure return new While((Expression) args[0], (Statement) args[1]);
				return lf.createWhile((Expression) args[0], (Statement) args[1]);
			}
		});
		put("Variable", new TemplateFactory() {
			public ILEntity create(Object... args) {
				return lf.createVariable((String) args[0]);
			}
		});

		put("Literal", new TemplateFactory() {
			public ILEntity create(Object... args) {
				if (args[0] instanceof Integer)
					return lf.createInt((Integer) args[0]);
				else
					return lf.createBoolean((Boolean) args[0]);
			}
		});

		put("GreaterThan", new TemplateFactory() {
			public ILEntity create(Object... args) {
				return lf.createGreaterThan((Expression) args[0], (Expression) args[1]);
			}
		});

		put("fact", new TemplateFactory() {
			public ILEntity create(Object... args) {
				Expression n = lf.createVariable("n");
				Expression result = lf.createVariable("result");
				Statement fatt = lf.createBlock(
						lf.createAssignment("n", lf.createInt((Integer) args[0])),
						lf.createAssignment("result", lf.createInt(1)),
						lf.createWhile(lf.createGreaterThan(n, lf.createInt(0)), lf.createBlock(
								lf.createAssignment("result", lf.createMultiplication(result, n)),
								lf.createAssignment("n",lf.createSubtraction(n, lf.createInt(1)))
						))
				);
				return fatt;
			}
		});
		//...
	}
}
