package labss.lang.il.iterators;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import labss.lang.il.model.ILEntity;

/**
 * @author Riccardo Solmi
 */
public class DescendantOrSelfIterator implements Iterator<ILEntity> {
	protected List<Iterator<ILEntity>> iteratorStack = new ArrayList<Iterator<ILEntity>>(16);

    public DescendantOrSelfIterator(ILEntity entity) {
    	this(entity, true);
    }
    public DescendantOrSelfIterator(ILEntity entity, boolean includeSelf) {
    	pushIterator(includeSelf ?
    			new SelfIterator(entity) : createChildIterator(entity));
    }

    protected Iterator<ILEntity> createChildIterator(ILEntity entity) {
    	return new ChildIterator(entity);
    }
    protected final Iterator<ILEntity> peekIterator() {
    	return iteratorStack.get(iteratorStack.size()-1);
    }
    protected final void pushIterator(Iterator<ILEntity> iterator) {
    	iteratorStack.add(iterator);
    }
    protected final void setIterator(Iterator<ILEntity> iterator) {
    	iteratorStack.set(iteratorStack.size()-1, iterator);
    }
    protected final Iterator<ILEntity> popIterator() {
    	return iteratorStack.remove(iteratorStack.size()-1);
    }

    public boolean hasNext() {
		return peekIterator().hasNext();
	}

	public ILEntity next() {
		ILEntity entity = peekIterator().next();
		
		if (entity.size() > 0)
			pushIterator(createChildIterator(entity));
		else
			prune();
		
		return entity;
	}

	public void prune() {
		while (iteratorStack.size()>1 && !peekIterator().hasNext())
			popIterator();
	}

	public void remove() {
		throw new UnsupportedOperationException();
	}
}
