package labss.lang.il.visitors;

import static labss.lang.il.model.ILPropertiesEnum.pCondition;
import static labss.lang.il.model.ILPropertiesEnum.pExp;
import static labss.lang.il.model.ILPropertiesEnum.pExp1;
import static labss.lang.il.model.ILPropertiesEnum.pExp2;
import static labss.lang.il.model.ILPropertiesEnum.pFalseBody;
import static labss.lang.il.model.ILPropertiesEnum.pTrueBody;
import static labss.lang.il.model.ILPropertiesEnum.pVariable;
import static labss.lang.il.model.ILTypesEnum.tAddition_ord;
import static labss.lang.il.model.ILTypesEnum.tAnd_ord;
import static labss.lang.il.model.ILTypesEnum.tAssignment_ord;
import static labss.lang.il.model.ILTypesEnum.tBlock_ord;
import static labss.lang.il.model.ILTypesEnum.tBool_ord;
import static labss.lang.il.model.ILTypesEnum.tDivision_ord;
import static labss.lang.il.model.ILTypesEnum.tEmpty_ord;
import static labss.lang.il.model.ILTypesEnum.tEquals_ord;
import static labss.lang.il.model.ILTypesEnum.tGreaterThan_ord;
import static labss.lang.il.model.ILTypesEnum.tIfElse_ord;
import static labss.lang.il.model.ILTypesEnum.tIf_ord;
import static labss.lang.il.model.ILTypesEnum.tInt_ord;
import static labss.lang.il.model.ILTypesEnum.tLessThan_ord;
import static labss.lang.il.model.ILTypesEnum.tMultiplication_ord;
import static labss.lang.il.model.ILTypesEnum.tNot_ord;
import static labss.lang.il.model.ILTypesEnum.tNullExpression_ord;
import static labss.lang.il.model.ILTypesEnum.tOr_ord;
import static labss.lang.il.model.ILTypesEnum.tSubtraction_ord;
import static labss.lang.il.model.ILTypesEnum.tVariable_ord;
import static labss.lang.il.model.ILTypesEnum.tWhile_ord;

import java.io.PrintStream;

import labss.lang.il.model.ILEntity;
import labss.lang.il.model.Literal;
import labss.lang.il.model.impl.Variable;

/**
 * @author Riccardo Solmi
 */
public class GenericPrettyPrintTypeSwitchBased {
	protected final PrintStream out;

	protected GenericPrettyPrintTypeSwitchBased(PrintStream out) {
		this.out = out;
	}

	public static void prettyPrint(ILEntity entity, PrintStream out) {
		new GenericPrettyPrintTypeSwitchBased(out).switchOn(entity);
	}

	public void switchOn(ILEntity entity) {
		switch (entity.getOrdinal()) {
		case tEmpty_ord:
			out.println(";");
			break;
		case tNullExpression_ord:
			out.print("null");
			break;
		case tWhile_ord:
			out.print("while (");
			switchOn(entity.get(pCondition));
			out.println(") {");
			switchOn(entity.get(pTrueBody));
			out.println("}");
			break;
		case tIf_ord:
			out.print("if (");
			switchOn(entity.get(pCondition));
			out.println(") {");
			switchOn(entity.get(pTrueBody));
			out.println("}");
			break;
		case tIfElse_ord:
			out.print("if (");
			switchOn(entity.get(pCondition));
			out.println(") {");
			switchOn(entity.get(pTrueBody));
			out.println("} else {");
			switchOn(entity.get(pFalseBody));
			out.println("}");
			break;
		case tAssignment_ord:
			switchOn(entity.get(pVariable));
			out.print(" = ");
			switchOn(entity.get(pExp));
			out.println(";");
			break;
		case tBlock_ord:
			for (int i = 0; i < entity.size(); i++)
				switchOn(entity.get(i));
			break;
		case tBool_ord:
			out.print(((Literal) entity).booleanValue());
			break;
		case tInt_ord:
			out.print(((Literal) entity).intValue());
			break;
		case tVariable_ord:
			out.print(((Variable) entity).getName());
			break;
		case tAddition_ord:
		case tSubtraction_ord:
		case tMultiplication_ord:
		case tDivision_ord:
		case tEquals_ord:
		case tGreaterThan_ord:
		case tLessThan_ord:
		case tAnd_ord:
		case tOr_ord:
			switchOn(entity.get(pExp1));
			out.print(entity.getType().opStr);
			switchOn(entity.get(pExp2));
			break;
		case tNot_ord:
			out.print(" ! ");
			switchOn(entity.get(pExp));
			break;
		}
	}
}
