package labss.lang.il.visitors;

import java.io.PrintStream;

import labss.lang.il.model.ILEntity;
import labss.lang.il.model.Literal;
import labss.lang.il.model.impl.Assignment;
import labss.lang.il.model.impl.BinaryExpression;
import labss.lang.il.model.impl.If;
import labss.lang.il.model.impl.IfElse;
import labss.lang.il.model.impl.Not;
import labss.lang.il.model.impl.Variable;
import labss.lang.il.model.impl.While;

/**
 * @author Riccardo Solmi
 */
public class PrettyPrintTypeSwitchBased5 {
	protected final PrintStream out;

	protected PrettyPrintTypeSwitchBased5(PrintStream out) {
		this.out = out;
	}

	public static void prettyPrint(ILEntity entity, PrintStream out) {
		new PrettyPrintTypeSwitchBased5(out).switchOn(entity);
	}

	public void switchOn(ILEntity entity) {
		switch (entity.getType5()) {
		case Empty:
			out.println(";");
			break;
		case NullExpression:
			out.print("null");
			break;
		case While:
			out.print("while (");
			switchOn(((While) entity).getCondition());
			out.println(") {");
			switchOn(((While) entity).getTrueBody());
			out.println("}");
			break;
		case If:
			out.print("if (");
			switchOn(((If) entity).getCondition());
			out.println(") {");
			switchOn(((If) entity).getTrueBody());
			out.println("}");
			break;
		case IfElse:
			out.print("if (");
			switchOn(((IfElse) entity).getCondition());
			out.println(") {");
			switchOn(((IfElse) entity).getTrueBody());
			out.println("} else {");
			switchOn(((IfElse) entity).getFalseBody());
			out.println("}");
			break;
		case Assignment:
			switchOn(((Assignment) entity).getVariable());
			out.print(" = ");
			switchOn(((Assignment) entity).getExp());
			out.println(";");
			break;
		case Block:
			for (int i = 0; i < entity.size(); i++)
				switchOn(entity.get(i));
			break;
		case Bool:
			out.print(((Literal) entity).booleanValue());
			break;
		case Int:
			out.print(((Literal) entity).intValue());
			break;
		case Variable:
			out.print(((Variable) entity).getName());
			break;
		case Addition:
		case Subtraction:
		case Multiplication:
		case Division:
		case Equals:
		case GreaterThan:
		case LessThan:
		case And:
		case Or:
			switchOn(((BinaryExpression) entity).getExp1());
			out.print(entity.getType().opStr);
			switchOn(((BinaryExpression) entity).getExp2());
			break;
		case Not:
			out.print(" ! ");
			switchOn(((Not) entity).getExp());
			break;
		}
	}
}
